library Backup;

uses
  Windows,
  Variants,
  SysUtils,
  Classes,
  System.Zip,
  Vcl.Dialogs,
  System.Types,
  System.IOUtils,
  Vcl.Forms,
  ExaPlugin_TLB in 'ExaPlugin_TLB.pas',
  Unit1 in 'Unit1.pas' {Form1};

{$R *.res}

type
  TPlugin = class(TInterfacedObject, IUnknown, IExaDataBaseCommandsPlugin)
  protected
    function Get_CommandCount: Integer; stdcall;
    function Get_Caption(Command: Integer): WideString; stdcall;
    procedure Execute(Command: Integer; const ExaApp: IExaApp); stdcall;
  end;

{ TPlugin }

procedure TPlugin.Execute(Command: Integer; const ExaApp: IExaApp);
var
  LPath, LFileName, LFile, LNewName: string;
  LFiles: TStringDynArray;
  LZipFile: TZipFile;
begin
  LFileName := FormatDateTime('yyyymmdd-hhnn', Now) + '.zip';
  try
    if PromptForFileName(LFileName, '*.zip|*.zip', 'zip',
      ',   ', '', True) then
    begin
      if TFile.Exists(LFileName) then
        TFile.Delete(LFileName);
      LPath := ExaApp.Get_DataBase.Get_Path;

      if Form1 = nil then
        Form1 := TForm1.Create(Application);
      SetWindowLong(Form1.Handle, GWL_HWNDPARENT, ExaApp.Get_MainFormHandle);
      Form1.ProgressBar.Position := 0;
      Form1.Show;
      try
        LFiles := TDirectory.GetFiles(LPath, '*', TSearchOption.soAllDirectories);
        Form1.ProgressBar.Max := Length(LFiles);

        LZipFile := TZipFile.Create;
        try
          LZipFile.Open(LFileName, zmWrite);
          for LFile in LFiles do
          begin
            Form1.ProgressBar.StepBy(1);
            Form1.Update;

            LNewName := LFile.Substring(LPath.Length);
            //  ,    ,    Backups
            if (not LNewName.Contains('\')) or LNewName.StartsWith('Backups\') then
              Continue;
            LZipFile.Add(LFile, LNewName);
          end;
          LZipFile.Close;
        finally
          LZipFile.Free;
        end;

      finally
        Form1.Hide;
      end;
    end;
  except
    Application.HandleException(Self);
  end;
end;

function TPlugin.Get_Caption(Command: Integer): WideString;
begin
  Result := '  ...';
end;

function TPlugin.Get_CommandCount: Integer;
begin
  Result := 1;
end;

{ mixed }

function Init: IInterface; safecall;
begin
  Result := TPlugin.Create;
end;

exports
  Init;

end.
